"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PgLtreePlugin = void 0;
const tslib_1 = require("tslib");
const pg_1 = require("@dataplan/pg");
const graphile_build_1 = require("graphile-build");
const pg_sql2_1 = tslib_1.__importDefault(require("pg-sql2"));
const version_js_1 = require("../version.js");
exports.PgLtreePlugin = {
    name: "PgLtreePlugin",
    version: version_js_1.version,
    gather: {
        initialState() {
            const ltreeCodec = (0, graphile_build_1.EXPORTABLE)((sql) => ({
                name: "ltree",
                sqlType: sql `ltree`,
                toPg(str) {
                    return str;
                },
                fromPg(str) {
                    return str;
                },
                executor: null,
                attributes: undefined,
            }), [pg_sql2_1.default]);
            const ltreeArrayCodec = (0, graphile_build_1.EXPORTABLE)((listOfCodec, ltreeCodec) => listOfCodec(ltreeCodec), [pg_1.listOfCodec, ltreeCodec]);
            return { ltreeCodec, ltreeArrayCodec };
        },
        hooks: {
            async pgCodecs_findPgCodec(info, event) {
                // If another plugin has already supplied a codec; skip
                if (event.pgCodec)
                    return;
                const { serviceName, pgType } = event;
                const ltreeExt = await info.helpers.pgIntrospection.getExtensionByName(serviceName, "ltree");
                if (!ltreeExt || pgType.typnamespace !== ltreeExt.extnamespace) {
                    return;
                }
                if (pgType.typname === "ltree") {
                    event.pgCodec = info.state.ltreeCodec;
                }
                else if (pgType.typname === "_ltree") {
                    event.pgCodec = info.state.ltreeArrayCodec;
                }
            },
        },
    },
    schema: {
        hooks: {
            init(_, build) {
                const codec = build.input.pgRegistry.pgCodecs.ltree;
                if (codec) {
                    const ltreeTypeName = build.inflection.builtin("LTree");
                    build.registerScalarType(ltreeTypeName, { pgCodec: codec }, () => ({
                        description: "Represents an `ltree` hierarchical label tree as outlined in https://www.postgresql.org/docs/current/ltree.html",
                        // TODO: specifiedByURL: https://postgraphile.org/scalars/ltree
                    }), 'Adding "LTree" scalar type from PgLtreePlugin.');
                    build.setGraphQLTypeForPgCodec(codec, "output", ltreeTypeName);
                    build.setGraphQLTypeForPgCodec(codec, "input", ltreeTypeName);
                }
                return _;
            },
        },
    },
};
//# sourceMappingURL=PgLtreePlugin.js.map